
/**************************************************************************
* This Code Runs the PMSM Servo motor connected to the MCHP MCLV Board
* The control signal fore the Servp is received over CAN bus from the HID CAN
* Node connected to a PC which Runs the Windows Control Software
* CAN Bus formed between PIC18F4550 and dsPIC33FJ128MC804  and PC over HID
****************************************************************************/
#include "built_in.h"
#include "appdef.h"
#include "IO_CAN.h"
#include "MotorControl.h"
//--------------------------------------------------------------------------
extern unsigned int HallValue;
extern unsigned int StateTableFwd[];
extern unsigned char CMD_Control;
extern int ActualSpeed;
//--------------------------------------------------------------------------
extern struct MotorFlags
{
unsigned RunMotor   :1;
unsigned Direction  :1;
unsigned isRunning  :1;
unsigned CloseLoop  :1;
unsigned swDirection :1;
unsigned unused     :11;
}Flags;

//---------------------------------------------------
// Main code
void main()
{
 /***************************************************************************
 * Set PLL : Fosc = ((Fin/PLLPRE)*PLLDIV)/PLLPOST ;
 * Configure Oscillator to operate the device at 20Mhz
 * Fosc= Fin*M/(N1*N2), Fcy=Fosc/2
 * Fosc= 8*10/(2*2)= 20Mhz for 8M input clock
 * Fin = 8Mhz
 * M = PLLFBD[PLLDIV<8:0>]   = 0x26  then M = 40  (PLLDIV + 2)
 * N1 = CLKDIV[PLLPRE<4:0>]  = 0x0   then N1 = 2  (PLLPRE + 2)
 * N2 = CLKDIV[PLLPOST<1:0>] = 0x3   then N2 = 8
******************************************************************************/
  PLLFBD =  8;                // M = 10
  CLKDIVbits.PLLPOST = 0;     // N1 = 2
  CLKDIVbits.PLLPRE = 0;      // N2 = 2
//-----------------------------------------------------
 // Initialize CAN module
//-----------------------------------------------------
  IO_CAN_Init();
//-----------------------------------------------------
// Initialize Motor Control Module
//------------------------------------------------------
  InitMCPWM();
//----------------------------------------------------
// Initialize ADC
//----------------------------------------------------
  InitADC10();

 //--------------------------------
 // Main Loop
 //--------------------------------
  while(1)
  {
  //----------------------------------------------
  // Get the operation Commands from PC Node /CAN
  //-----------------------------------------------
    IO_CanRead();
  //------------------------------------------------
  // Control the Servo motor as requested by PC
  //-------------------------------------------------
   Flags.RunMotor = (CMD_Control & 0x01);
   Flags.swDirection = ((CMD_Control>>1) & 0x01);
   if(Flags.RunMotor)
   {
     if((!Flags.swDirection)&&(!Flags.isRunning))
        {StartMotorFWD();Flags.Direction = 1;}
     else if((Flags.swDirection)&&(!Flags.isRunning))
        {StartMotorREV();Flags.Direction = 0;}
   }
   else if(!Flags.RunMotor){StopMotor();}
   //-------------------------------------------------------
   // if motor has started select Close loop speed control
   //--------------------------------------------------------
   if(ActualSpeed >100){Flags.CloseLoop = 1;}
    else{Flags.CloseLoop = 0;}
    Delay_ms(20);
  //-------------------------------------------------
  // Write Servo Controller Status to PC Node
  //-------------------------------------------------
    IO_CanWrite();
  }
}
//---------------------------------------------------------------------------