//----------------------------------------------------------------------
// IO_CAN.c
//-----------------------------------------------------------------------
#include "IO_CAN.h"
#include "ECAN_Defs.h"
//-----------------------------------------------------------------------
unsigned int Can_Init_Flags, Can_Send_Flags, Can_Rcv_Flags;  // can flags
unsigned int RxDataLen;     // received data length in bytes
unsigned int TxDataLen;
char RxData[8];            // can rx/tx data buffer
char TxData[8];
char Msg_Rcvd;             // reception flag
unsigned long RxID;
unsigned long TxID;
//------------------------------------------------------------------------
extern int ActualSpeed;
extern int DesiredSpeed;
unsigned char CMD_Control = 0;
unsigned char ServoStatus = 0;
unsigned char MotTemperature = 0;
extern struct MotorFlags
{
unsigned RunMotor   :1;
unsigned Direction  :1;
unsigned isRunning  :1;
unsigned CloseLoop :1;
unsigned swDirection :1;
unsigned unused     :11;
}Flags;

//------------------------------------------------------------------------
// CAN ISR
//----------------------------------------------------------------------
void C1Interrupt(void) org 0x005A  // ECAN event iterrupt
{

  IFS2bits.C1IF = 0;      // clear ECAN interrupt flag
  if(C1INTFbits.TBIF)
  {                       // was it tx interrupt?
    C1INTFbits.TBIF = 0;  // if yes clear tx interrupt flag
  }

  if(C1INTFbits.RBIF)
   {                       // was it rx interrupt?
     C1INTFbits.RBIF = 0;  // if yes clear rx interrupt flag
   }
}
//-----------------------------------------------------------------------
void IO_CanRead()
{
  Msg_Rcvd = ECAN1Read(&RxID,RxData,&RxDataLen,&Can_Rcv_Flags);
  if(Msg_Rcvd!=0)
  {
    switch (RxID)
   {
     case 0x211 :
      CMD_Control = RxData[0];
      DesiredSpeed = (RxData[1]* 40);
      break;
     case 0x310:
       asm nop;
      break;
     case 0x410 :
      asm nop;
     break;

   }// end switch case
  
  }
}
//---------------------------------------------------------------------
void IO_CanWrite()
{
  ServoStatus = CMD_Control;
  TxData[0] = ServoStatus;
  TxData[1] = (ActualSpeed &0xFF);
  TxData[2] = (ActualSpeed >> 8);
  TxData[3] = (MotTemperature + 40);
  TxID = 0x210;
  ECAN1Write(TxID,TxData,4,Can_Send_Flags);

}
//----------------------------------------------------------------------
void IO_CAN_Init()
{
 AD1PCFGL = 0xFFFF;      // PORTB pins as Digital
  // Check TRISB for correct sewttings

//-----------------------------------------------------
// Configure IC Pins for CAN Tx and Rx
//-----------------------------------------------------
  Unlock_IOLOCK();
  PPS_Mapping_NoLock(9,_OUTPUT,_C1TX);
  PPS_Mapping_NoLock(7,_INPUT,_CIRX);
  Lock_IOLOCK();
  
 /* Clear Interrupt Flags */
  IFS0=0;
  IFS1=0;
  IFS2=0;
  IFS3=0;
  IFS4=0;

  /* Enable ECAN1 Interrupt */
  IEC2bits.C1IE   = 1;             // enable ECAN1 interrupts
  C1INTEbits.TBIE = 1;            // enable ECAN1 tx interrupt
  C1INTEbits.RBIE = 1;            // enable ECAN1 rx interrupt
  Can_Init_Flags = 0;             //
  Can_Send_Flags = 0;             // clear flags
  Can_Rcv_Flags  = 0;             //

  Can_Send_Flags = _ECAN_TX_PRIORITY_0 &
                   _ECAN_TX_STD_FRAME &
                   _ECAN_TX_NO_RTR_FRAME;

  Can_Init_Flags = _ECAN_CONFIG_SAMPLE_THRICE &
                   _ECAN_CONFIG_PHSEG2_PRG_ON &
                   _ECAN_CONFIG_STD_MSG &
                   _ECAN_CONFIG_MATCH_MSG_TYPE &
                   _ECAN_CONFIG_LINE_FILTER_OFF;

  // init dma channel 0 for  dma to ECAN peripheral transfer
  ECAN1DmaChannelInit(0, 1, &ECAN1RxTxRAMBuffer);
  // init dma channel 2 for ECAN peripheral to dma transfer
  ECAN1DmaChannelInit(2, 0, &ECAN1RxTxRAMBuffer);
   // Set Baud Rate  500Kb/sec@20MHz
  ECAN1Initialize(1, 1, 3, 3, 3, Can_Init_Flags);
  // number of rx+tx buffers in DMA RAM
  ECAN1SetBufferSize(ECAN1RAMBUFFERSIZE);
   // select transmit buffers 0x000F = buffers 0:3 are transmit buffers
  ECAN1SelectTxBuffers(0x000F);
  // set CONFIGURATION mode
  ECAN1SetOperationMode(_ECAN_MODE_CONFIG,0xFF);
  ECAN1SetMask(_ECAN_MASK_0, -1,_ECAN_CONFIG_MATCH_MSG_TYPE &
               _ECAN_CONFIG_STD_MSG);     // set all mask1 bits to ones
  ECAN1SetMask(_ECAN_MASK_1,-1,_ECAN_CONFIG_MATCH_MSG_TYPE &
               _ECAN_CONFIG_STD_MSG);    // set all mask2 bits to ones
  ECAN1SetMask(_ECAN_MASK_2,-1,_ECAN_CONFIG_MATCH_MSG_TYPE & 
                _ECAN_CONFIG_STD_MSG);  // set all mask3 bits to ones
  ECAN1SetFilter(_ECAN_FILTER_10,0x211,_ECAN_MASK_2,_ECAN_RX_BUFFER_7,
                _ECAN_CONFIG_STD_MSG);  // set id of filter10
  ECAN1SetOperationMode(_ECAN_MODE_NORMAL, 0xFF); // set NORMAL mode
  Msg_Rcvd = 0;
}
//********************************EOF****************************************