
//----------------------------------------------------------------------
//  MotorControl.c
//-----------------------------------------------------------------------
#include "MotorControl.h"
//-----------------------------------------------------------------------
unsigned int HallValue;
unsigned int timer3value;
unsigned int timer3avg;
int DesiredSpeed;
int ActualSpeed;
int SpeedError;
long SpeedIntegral = 0, SpeedIntegral_n_1 = 0, SpeedProportional = 0;
long DutyCycle = 0;
unsigned int Kps = 20000;// Kp and Ks terms need to be adjusted
unsigned int Kis = 2000;// as per the motor and load
unsigned char polecount;
//------------------------------------------------------------------------
#define FOSC  20000000             // xtal = 8.0Mhz, 20.0Mhz after PLL
#define FCY  FOSC/2
#define MILLISEC FCY/20000          // 1 mSec delay constant
#define FPWM 39000
#define POLEPAIRS        5          // number of pole pairs
#define HALL_INDEX_R     4         // Hall sensor position index
#define HALL_INDEX_F     5         // Hall sensor position index

//#define CLOSEDLOOP       //uncomment this line to open loop
#define SPEEDMULT  2343750
#define OFFSET     8
#define T1PR1 ((FCY/1000)/64)
//-----------------------------------------------------------------------
// Commutation Tables
//-----------------------------------------------------------------------
unsigned int StateTableFwd[] = {0x0000, 0x0210, 0x2004, 0x0204,
                                0x0801, 0x0810, 0x2001, 0x0000};
unsigned int StateTableRev[] = {0x0000, 0x2001, 0x0810, 0x0801,
                                0x0204, 0x2004, 0x0210, 0x0000};
//---------------------------------------------------------------------
// CAN variables
//----------------------------------------------------------------------
extern unsigned char MotTemperature;
//----------------------------------------------------------------------
// Control Flags
//----------------------------------------------------------------------
struct MotorFlags
{
unsigned RunMotor   :1;
unsigned Direction  :1;
unsigned isRunning  :1;
unsigned CloseLoop :1;
unsigned swDirection :1;
unsigned unused     :11;
}Flags;

//-----------------------------------------------------------------------
// Timer 1 ISR
//-------------------------------------------------------
 void Timer1Int() iv IVT_ADDR_T1INTERRUPT
{
  T1IF_bit = 0;          // clear T1IF
  ActualSpeed = SPEEDMULT/timer3avg;
if(Flags.CloseLoop)
{
  SpeedError = DesiredSpeed - ActualSpeed;
  SpeedProportional = (int)(((long)Kps*(long)SpeedError) >> 15);
  SpeedIntegral = SpeedIntegral_n_1 + (int)(((long)Kis*(long)SpeedError) >> 15);

  if (SpeedIntegral < 0)
       SpeedIntegral = 0;
   else if (SpeedIntegral > 32767)
       SpeedIntegral = 32767;
       SpeedIntegral_n_1 = SpeedIntegral;
       DutyCycle = SpeedIntegral + SpeedProportional;
  if (DutyCycle < 0)
       DutyCycle = 0;
  else if (DutyCycle > 32767)
           DutyCycle = 32767;

   PDC1 = (int)(((long)(PTPER*2)*(long)DutyCycle) >> 15);
   PDC2 = PDC1;
   PDC3 = PDC1;
 }               // in closed loop algorithm
  if(Flags.RunMotor == 0) { ActualSpeed = 0;}

}
//--------------------------------------------------------------------------
// IC1,IC2 and IC7 Int handle the Hall sensor interrupts for Commutation
//--------------------------------------------------------------------------
void IC1Int() iv IVT_ADDR_IC1INTERRUPT
{
  int Hall_Index;
  IFS0bits.IC1IF = 0;        // Clear interrupt flag
  HallValue = (unsigned int)((PORTB >> 1) & 0x0007);        // Read halls

if (Flags.Direction)
 {
   OVDCON = StateTableFwd[HallValue];
   Hall_Index = HALL_INDEX_F;
 }
else
 {
   OVDCON = StateTableRev[HallValue];
   Hall_Index = HALL_INDEX_R;
 }

// The code below is uses TMR3 to calculate the speed of the rotor
   if (HallValue == Hall_Index)  // has the same position been sensed?
   if (polecount++ == POLEPAIRS)  //has one mech rev elasped?
   {                              // yes then read timer 3
     timer3value = TMR3;
     TMR3 = 0;
     timer3avg = ((timer3avg + timer3value) >> 1);
     polecount = 1;
   }
}
//-------------------------------------------------------------------------
void IC2Int() iv IVT_ADDR_IC2INTERRUPT
{
  IFS0bits.IC2IF = 0;        // Clear interrupt flag
  HallValue = (unsigned int)((PORTB >> 1) & 0x0007);        // Read halls

  if (Flags.Direction)
      OVDCON = StateTableFwd[HallValue];
  else
      OVDCON = StateTableRev[HallValue];
}
//------------------------------------------------------------------------
void IC7Int() iv IVT_ADDR_IC7INTERRUPT
{
  IFS1bits.IC7IF = 0;        // Clear interrupt flag
  HallValue = (unsigned int)((PORTB >> 1) & 0x0007);        // Read halls
 if (Flags.Direction)
    OVDCON = StateTableFwd[HallValue];
 else
    OVDCON = StateTableRev[HallValue];
}
//--------------------------------------------------------------------------
void AdcInt() iv IVT_ADDR_ADC1INTERRUPT
{
 IFS0bits.AD1IF = 0;
 // Read Pot on  AN8 to simulate  mot temperature
  MotTemperature =(ADC1BUF0/5);
}
//----------------------------------------------------------------------
// Start Motor and Run FWD Direction
//----------------------------------------------------------------------
void StartMotorFWD()
{
  PWMCON1 = 0x0777;        // enable PWM outputs
  HallValue = (unsigned int)((PORTB >> 1) & 0x0007); //Read Rotor Position
  OVDCON = StateTableFwd[HallValue];// Switch phases to start motor
  PDC1 = PDC2 = PDC3 = 200;  //Force commutation
  T3CONbits.TON = 1;     // start tmr3 for speed calculation
  polecount = 1;
  Flags.isRunning = 1;
}
//----------------------------------------------------------------------
// StartMotor and Run REV Direction
//----------------------------------------------------------------------
void StartMotorREV()
{
  PWMCON1 = 0x0777;        // enable PWM outputs
  HallValue = (unsigned int)((PORTB >> 1) & 0x0007); //Read Rotor Position
  OVDCON = StateTableRev[HallValue];// Switch phases to start motor
  PDC1 = PDC2 = PDC3 = 200; // Force commutation
  T3CONbits.TON = 1;     // start tmr3 for speed calculation
  polecount = 1;
  Flags.isRunning = 1;
}
//----------------------------------------------------------------------
// Stop Motor
//----------------------------------------------------------------------
void  StopMotor(void)
{
  PDC1 = PDC2 = PDC3 = 100;
  PWMCON1 = 0x0700;       // disable PWMs
  Flags.CloseLoop = 0; // Use speed control PID
  Flags.isRunning = 0;
}
//-----------------------------------------------------------------------
void InitMCPWM(void)
{
 //-----------------------------------------------------
// Configure IC Pins for Hall Sensors
//-----------------------------------------------------
  Unlock_IOLOCK();
  PPS_Mapping_NoLock(1,_INPUT ,_IC1);
  PPS_Mapping_NoLock(2,_INPUT ,_IC2);
  PPS_Mapping_NoLock(3,_INPUT ,_IC7);
  Lock_IOLOCK();
 //----------------------------------------------------
 // Set Pin Directions
 //-----------------------------------------------------
  TRISBbits.TRISB15 = 0;
  TRISBbits.TRISB14 = 0;
  TRISBbits.TRISB13 = 0;
  TRISBbits.TRISB12 = 0;
  TRISBbits.TRISB11 = 0;
  TRISBbits.TRISB10 = 0;
  TRISBbits.TRISB3  = 1;
  TRISBbits.TRISB2  = 1;
 //----------------------------------------------------
 // Config Timer1
 //--------------------------------------------------------------
  // MIPs / (Scale * Hz interrupt)
  // For this design, 8MHz * 2.5PLL, = 10 MIP
  // 10MIP/(64 * 1000) = 156, => PR1 = 156 for 1000Hz interrupt
  //-------------------------------------------------------------
  T1CON = 0x8020;      // internal Tcy/64 clock
  TMR1 = 0;
  PR1 = T1PR1;
  T1CONbits.TON = 1;  // turn on timer 1
  IFS0bits.T1IF = 0;
  IEC0bits.T1IE = 1;
//-----------------------------------------------------------------
// Configure Timer 3
//------------------------------------------------------------------
  T3CON = 0x0030;           // internal Tcy/256 clock
  TMR3 = 0;
  PR3 = 0xFFFF;
 //----------------------------------------------------------------------------
// Configure IC Pins
//----------------------------------------------------------------------------
  //Hall A -> IC1. Hall A is used for Speed measurement and commutation.
  //Hall B -> IC2. Hall B is only used for commutation.
  //Hall C -> IC3. Hall C is only used for commutation.
   IC1CON = 1;        // Init all 3 Hall Effect capture inputs:
   IC2CON = 1;        // Timer 3, every capture event, rise & fall edges
   IC7CON = 1;

  IFS0bits.IC1IF = 0;     // Clear interrupt flag
  IFS0bits.IC2IF = 0;
  IFS1bits.IC7IF = 0;
  IEC0bits.IC1IE = 1;    // Enable interrupt
  IEC0bits.IC2IE = 1;
  IEC1bits.IC7IE = 1;
 //----------------------------------------------------
   PTPER = FCY/FPWM - 1;
   PWMCON1 = 0x0700;       // disable PWMs
   OVDCON = 0x0000;        // allow control using OVD
   PDC1 = 100;             // init PWM 1, 2 and 3 to 100
   PDC2 = 100;
   PDC3 = 100;
   SEVTCMP = PTPER;
   PWMCON2 = 0x0F00;      // 16 postscale values
   PTCON = 0x8000;        // start PWM
   //---------------------------------------------------
   // Ensure that ControlFlags are set to 0 in startup
   //---------------------------------------------------
   Flags.RunMotor = 0;
   Flags.Direction = 0;
   Flags.isRunning = 0;
   ActualSpeed = 0;
}
//------------------------------------------------------------------------
void InitADC10(void)
{
   AD1PCFGLbits.PCFG8 = 0;         // ensure AN8 pot
 /* set channel scanning here, auto sampling and convert,
    with default read-format mode */
   AD1CON1 = 0x006C;
/* select 10-bit, 1 channel ADC operation */
   AD1CON1bits.AD12B = 0;
/* No channel scan for CH0+, Use MUX A,
   SMPI = 1 per interrupt, Vref = AVdd/AVss */
   AD1CON2 = 0x0000;
/* Set Samples and bit conversion time */
   AD1CON3 = 0x032F;
   AD1CSSL = 0x0000;
/* channel select AN8 */
   AD1CHS0 = 0x0008;
   IFS0bits.AD1IF = 0;
   IEC0bits.AD1IE = 1;
   AD1CON1bits.ADON = 1;
}

//-------------------------------------------------------------------------